<?php
namespace wcf\system\event\listener;
use wcf\acp\form\UserGroupAddForm;
use wcf\acp\form\UserGroupEditForm;
use wcf\data\package\PackageCache;
use wcf\data\user\group\UserGroup;
use wcf\data\user\group\UserGroupEditor;
use wcf\data\user\UserList;
use wcf\system\event\IEventListener;
use wcf\system\exception\UserInputException;
use wcf\system\language\I18nHandler;
use wcf\system\WCF;
use wcf\util\ArrayUtil;
use wcf\util\StringUtil;

/**
 * Handles groupType and description.
 * 
 * @author	Tim Duesterhus
 * @copyright	2001-2013 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wcf.moderatedUserGroup
 * @subpackage	system.event.listener
 * @category	Community Framework (commercial)
 */
class UserGroupAddFormModeratedGroupListener implements IEventListener {
	/**
	 * instance of UserGroupAddForm
	 * @var	wcf\acp\form\UserGroupAddForm
	 */
	protected $eventObj = null;
	
	/**
	 * group type
	 * @var	integer
	 */
	protected $groupType = 4;
	
	/**
	 * group description
	 * @var	string
	 */
	protected $description = '';
	
	/**
	 * array of leader given by the user
	 * @var	array<string>
	 */
	protected $leader = array();
	
	/**
	 * userlist containing the group leaders
	 * @var	wcf\data\user\UserList
	 */
	protected $leaderList = null;
	
	/**
	 * is the eventlistener disabled for this group
	 * @var	boolean
	 */
	private $disable = false;
	
	/**
	 * packageID of com.woltlab.wcf.moderatedUserGroup
	 * @var	integer
	 */
	private $packageID = null;
	
	/**
	 * @see	wcf\system\event\IEventListener::execute()
	 */
	public function execute($eventObj, $className, $eventName) {
		$this->eventObj = $eventObj;
		
		if ($this->packageID === null) {
			// read packageID as it is needed for i18n
			$this->packageID = PackageCache::getInstance()->getPackageID('com.woltlab.wcf.moderatedUserGroup');
		}
		
		if ($this->eventObj instanceof UserGroupEditForm && is_object($this->eventObj->group)) {
			switch ($this->eventObj->group->groupType) {
				// default groups must have the default groupType
				case UserGroup::EVERYONE:
				case UserGroup::GUESTS:
				case UserGroup::USERS:
					$this->disable = true;
				break;
				
				default:
					I18nHandler::getInstance()->register('description');
				break;
			}
		}
		else if ($this->eventObj instanceof UserGroupAddForm) {
			I18nHandler::getInstance()->register('description');
		}
		
		if ($this->disable) return;
		$this->$eventName();
	}
	
	/**
	 * Handles the assignVariables event.
	 */
	protected function assignVariables() {
		WCF::getTPL()->assign(array(
			'groupType' => $this->groupType,
			'description' => $this->description,
			'leader' => implode(', ', $this->leader)
		));
	}
	
	/**
	 * Handles the readData event.
	 * This is only called in UserGroupEditForm.
	 */
	protected function readData() {
		if (empty($_POST)) {
			I18nHandler::getInstance()->setOptions('description', $this->packageID, $this->eventObj->group->groupDescription, 'wcf.acp.group.description\d+');
			$this->groupType = $this->eventObj->group->groupType;
			$this->description = $this->eventObj->group->groupDescription;
			
			$this->leaderList = new UserList();
			$this->leaderList->getConditionBuilder()->add('user_table.userID IN(
				SELECT	leaderID 
				FROM	wcf'.WCF_N.'_user_group_leader 
				WHERE	groupID = ?
			)', array($this->eventObj->group->groupID));
			$this->leaderList->sqlOrderBy = 'user_table.username ASC';
			$this->leaderList->readObjects();
			
			$this->leader = array();
			foreach ($this->leaderList as $user) {
				$this->leader[] = $user->username;
			}
		}
	}
	
	/**
	 * Handles the readFormParameters event.
	 */
	protected function readFormParameters() {
		if (isset($_POST['groupType'])) $this->groupType = intval($_POST['groupType']);
		if (isset($_POST['leader'])) $leaders = ArrayUtil::trim(explode(',', $_POST['leader']));
		// remove duplicates
		foreach ($leaders as $leader) $this->leader[StringUtil::toLowerCase($leader)] = $leader;
		
		I18nHandler::getInstance()->readValues();
		
		if (I18nHandler::getInstance()->isPlainValue('description')) $this->description = I18nHandler::getInstance()->getValue('description');
	}
	
	/**
	 * Handles the save event.
	 */
	protected function save() {
		if ($this->eventObj instanceof UserGroupEditForm) {
			// save i18n values
			$this->description = 'wcf.acp.group.description'.$this->eventObj->group->groupID;
			if (I18nHandler::getInstance()->isPlainValue('description')) {
				I18nHandler::getInstance()->remove($this->description, $this->packageID);
				$this->description = I18nHandler::getInstance()->getValue('description');
			}
			else {
				I18nHandler::getInstance()->save('description', $this->description, 'wcf.acp.group', $this->packageID);
			}
			
			$this->eventObj->additionalFields = array_merge($this->eventObj->additionalFields, array(
				'groupType' => $this->groupType,
				'groupDescription' => $this->description
			));
			
			// save group leaders
			$sql = "DELETE FROM wcf".WCF_N."_user_group_leader WHERE groupID = ?";
			$statement = WCF::getDB()->prepareStatement($sql);
			$statement->execute(array($this->eventObj->group->groupID));
			
			if ($this->leaderList !== null) {
				$sql = "INSERT INTO wcf".WCF_N."_user_group_leader (groupID, leaderID) VALUES (?, ?)";
				$statement = WCF::getDB()->prepareStatement($sql);
				foreach ($this->leaderList as $user) {
					$statement->execute(array($this->eventObj->group->groupID, $user->userID));
				}
			}
			
			// delete application when groupType is changed to a type that does not support them
			// moderated, closed moderated
			if ($this->groupType != 6 && $this->groupType != 7) {
				$sql = "DELETE FROM wcf".WCF_N."_user_group_application WHERE groupID = ?";
				$statement = WCF::getDB()->prepareStatement($sql);
				$statement->execute(array($this->eventObj->group->groupID));
			}
		}
		else {
			$this->eventObj->additionalFields = array_merge($this->eventObj->additionalFields, array(
				'groupType' => $this->groupType,
				'groupDescription' => $this->description
			));
		}
	}
	
	/**
	 * Handles the saved event.
	 * This is only called in UserGroupAddForm.
	 */
	protected function saved() {
		$returnValues = $this->eventObj->objectAction->getReturnValues();
		$groupID = $returnValues['returnValues']->groupID;
		
		if (!I18nHandler::getInstance()->isPlainValue('description')) {
			I18nHandler::getInstance()->save('description', 'wcf.acp.group.description'.$groupID, 'wcf.acp.group', 1);
			
			// update group description
			$groupEditor = new UserGroupEditor($returnValues['returnValues']);
			$groupEditor->update(array(
				'groupDescription' => 'wcf.acp.group.description'.$groupID
			));
		}
		
		if ($this->leaderList !== null) {
			// save group leaders
			$sql = "INSERT INTO wcf".WCF_N."_user_group_leader (groupID, leaderID) VALUES (?, ?)";
			$statement = WCF::getDB()->prepareStatement($sql);
			foreach ($this->leaderList as $user) {
				$statement->execute(array($groupID, $user->userID));
			}
		}
	}
	
	/**
	 * Handles the validate event.
	 */
	protected function validate() {
		try {
			if ($this->groupType > 7 || $this->groupType < 4) {
				throw new UserInputException('groupType', 'notValid');
			}
		}
		catch (UserInputException $e) {
			$this->eventObj->errorType[$e->getField()] = $e->getType();
		}
		
		try {
			if (!empty($this->leader)) {
				$this->leaderList = new UserList();
				$this->leaderList->getConditionBuilder()->add('username IN (?)', array($this->leader));
				$this->leaderList->readObjects();
				
				$tmp = array();
				foreach ($this->leaderList as $user) {
					$tmp[StringUtil::toLowerCase($user->username)] = $user->username;
				}
				
				$difference = array_diff_key($this->leader, $tmp);
				if (!empty($difference)) {
					WCF::getTPL()->assign(array(
						'unknownLeader' => $difference
					));
					throw new UserInputException('leader', 'notFound');
				}
				
				$this->leader = $tmp;
			}
			
			// moderated, closed moderated
			if (($this->groupType == 6 || $this->groupType == 7) && empty($this->leader)) {
				throw new UserInputException('leader');
			}
		}
		catch (UserInputException $e) {
			$this->eventObj->errorType[$e->getField()] = $e->getType();
		}
	}
}
